#!/usr/bin/env python
# coding=UTF-8
#
# Generated by pykdeuic4 from folderpage.ui on Sun Mar  4 11:39:39 2012
#
# WARNING! All changes to this file will be lost.
from PyKDE4 import kdecore
from PyKDE4 import kdeui
from PyQt4 import QtCore, QtGui

try:
    _fromUtf8 = QtCore.QString.fromUtf8
except AttributeError:
    _fromUtf8 = lambda s: s

class Ui_FolderPage(object):
    def setupUi(self, FolderPage):
        FolderPage.setObjectName(_fromUtf8("FolderPage"))
        FolderPage.resize(568, 530)
        self.verticalLayout_2 = QtGui.QVBoxLayout(FolderPage)
        self.verticalLayout_2.setObjectName(_fromUtf8("verticalLayout_2"))
        self.urlLabel = KUrlLabel(FolderPage)
        self.urlLabel.setAlignment(QtCore.Qt.AlignCenter)
        self.urlLabel.setObjectName(_fromUtf8("urlLabel"))
        self.verticalLayout_2.addWidget(self.urlLabel)
        self.settingsGroupBox = QtGui.QGroupBox(FolderPage)
        self.settingsGroupBox.setObjectName(_fromUtf8("settingsGroupBox"))
        self.verticalLayout = QtGui.QVBoxLayout(self.settingsGroupBox)
        self.verticalLayout.setObjectName(_fromUtf8("verticalLayout"))
        self.showInTrayCheckbox = QtGui.QCheckBox(self.settingsGroupBox)
        self.showInTrayCheckbox.setObjectName(_fromUtf8("showInTrayCheckbox"))
        self.verticalLayout.addWidget(self.showInTrayCheckbox)
        self.kseparator = KSeparator(self.settingsGroupBox)
        self.kseparator.setObjectName(_fromUtf8("kseparator"))
        self.verticalLayout.addWidget(self.kseparator)
        self.settingsWidget = SettingsWidget(self.settingsGroupBox)
        self.settingsWidget.setObjectName(_fromUtf8("settingsWidget"))
        self.verticalLayout.addWidget(self.settingsWidget)
        self.verticalLayout_2.addWidget(self.settingsGroupBox)
        spacerItem = QtGui.QSpacerItem(20, 40, QtGui.QSizePolicy.Minimum, QtGui.QSizePolicy.Expanding)
        self.verticalLayout_2.addItem(spacerItem)

        self.retranslateUi(FolderPage)
        QtCore.QMetaObject.connectSlotsByName(FolderPage)

    def retranslateUi(self, FolderPage):
        FolderPage.setWindowTitle(kdecore.i18n(_fromUtf8("Form")))
        self.urlLabel.setTipText(kdecore.i18n(_fromUtf8("Open the folder in the default file manager")))
        self.settingsGroupBox.setTitle(kdecore.i18n(_fromUtf8("Folder Settings")))
        self.showInTrayCheckbox.setText(kdecore.i18n(_fromUtf8("Show in notification icon menu")))

from PyKDE4.kdeui import KUrlLabel, KSeparator
from .configwindow import SettingsWidget
