#!/usr/bin/env python
# coding=UTF-8
#
# Generated by pykdeuic4 from scriptpage.ui on Sun Mar  4 11:39:40 2012
#
# WARNING! All changes to this file will be lost.
from PyKDE4 import kdecore
from PyKDE4 import kdeui
from PyQt4 import QtCore, QtGui

try:
    _fromUtf8 = QtCore.QString.fromUtf8
except AttributeError:
    _fromUtf8 = lambda s: s

class Ui_ScriptPage(object):
    def setupUi(self, ScriptPage):
        ScriptPage.setObjectName(_fromUtf8("ScriptPage"))
        ScriptPage.resize(587, 581)
        self.verticalLayout_2 = QtGui.QVBoxLayout(ScriptPage)
        self.verticalLayout_2.setObjectName(_fromUtf8("verticalLayout_2"))
        self.urlLabel = KUrlLabel(ScriptPage)
        self.urlLabel.setAlignment(QtCore.Qt.AlignCenter)
        self.urlLabel.setObjectName(_fromUtf8("urlLabel"))
        self.verticalLayout_2.addWidget(self.urlLabel)
        self.scriptCodeEditor = Qsci.QsciScintilla(ScriptPage)
        self.scriptCodeEditor.setToolTip(_fromUtf8(""))
        self.scriptCodeEditor.setWhatsThis(_fromUtf8(""))
        self.scriptCodeEditor.setObjectName(_fromUtf8("scriptCodeEditor"))
        self.verticalLayout_2.addWidget(self.scriptCodeEditor)
        self.settingsGroupbox = QtGui.QGroupBox(ScriptPage)
        self.settingsGroupbox.setObjectName(_fromUtf8("settingsGroupbox"))
        self.verticalLayout = QtGui.QVBoxLayout(self.settingsGroupbox)
        self.verticalLayout.setObjectName(_fromUtf8("verticalLayout"))
        self.promptCheckbox = QtGui.QCheckBox(self.settingsGroupbox)
        self.promptCheckbox.setObjectName(_fromUtf8("promptCheckbox"))
        self.verticalLayout.addWidget(self.promptCheckbox)
        self.showInTrayCheckbox = QtGui.QCheckBox(self.settingsGroupbox)
        self.showInTrayCheckbox.setObjectName(_fromUtf8("showInTrayCheckbox"))
        self.verticalLayout.addWidget(self.showInTrayCheckbox)
        self.kseparator = KSeparator(self.settingsGroupbox)
        self.kseparator.setObjectName(_fromUtf8("kseparator"))
        self.verticalLayout.addWidget(self.kseparator)
        self.settingsWidget = SettingsWidget(self.settingsGroupbox)
        self.settingsWidget.setObjectName(_fromUtf8("settingsWidget"))
        self.verticalLayout.addWidget(self.settingsWidget)
        self.verticalLayout_2.addWidget(self.settingsGroupbox)

        self.retranslateUi(ScriptPage)
        QtCore.QMetaObject.connectSlotsByName(ScriptPage)

    def retranslateUi(self, ScriptPage):
        ScriptPage.setWindowTitle(kdecore.i18n(_fromUtf8("Form")))
        self.urlLabel.setTipText(kdecore.i18n(_fromUtf8("Open the script in the default text editor")))
        self.settingsGroupbox.setTitle(kdecore.i18n(_fromUtf8("Script Settings")))
        self.promptCheckbox.setText(kdecore.i18n(_fromUtf8("Always prompt before running this script")))
        self.showInTrayCheckbox.setText(kdecore.i18n(_fromUtf8("Show in notification icon menu")))

from PyQt4 import Qsci
from PyKDE4.kdeui import KUrlLabel, KSeparator
from .configwindow import SettingsWidget
