#!/usr/bin/env python
# coding=UTF-8
#
# Generated by pykdeuic4 from settingswidget.ui on Sun Mar  4 11:39:40 2012
#
# WARNING! All changes to this file will be lost.
from PyKDE4 import kdecore
from PyKDE4 import kdeui
from PyQt4 import QtCore, QtGui

try:
    _fromUtf8 = QtCore.QString.fromUtf8
except AttributeError:
    _fromUtf8 = lambda s: s

class Ui_SettingsWidget(object):
    def setupUi(self, SettingsWidget):
        SettingsWidget.setObjectName(_fromUtf8("SettingsWidget"))
        SettingsWidget.resize(316, 91)
        self.gridLayout = QtGui.QGridLayout(SettingsWidget)
        self.gridLayout.setObjectName(_fromUtf8("gridLayout"))
        self.label = QtGui.QLabel(SettingsWidget)
        self.label.setObjectName(_fromUtf8("label"))
        self.gridLayout.addWidget(self.label, 0, 0, 1, 1)
        self.abbrLabel = QtGui.QLabel(SettingsWidget)
        self.abbrLabel.setObjectName(_fromUtf8("abbrLabel"))
        self.gridLayout.addWidget(self.abbrLabel, 0, 1, 1, 1)
        self.setAbbrButton = KPushButton(SettingsWidget)
        self.setAbbrButton.setObjectName(_fromUtf8("setAbbrButton"))
        self.gridLayout.addWidget(self.setAbbrButton, 0, 3, 1, 1)
        self.clearAbbrButton = KPushButton(SettingsWidget)
        self.clearAbbrButton.setObjectName(_fromUtf8("clearAbbrButton"))
        self.gridLayout.addWidget(self.clearAbbrButton, 0, 4, 1, 1)
        self.label_2 = QtGui.QLabel(SettingsWidget)
        self.label_2.setObjectName(_fromUtf8("label_2"))
        self.gridLayout.addWidget(self.label_2, 1, 0, 1, 1)
        self.hotkeyLabel = QtGui.QLabel(SettingsWidget)
        self.hotkeyLabel.setObjectName(_fromUtf8("hotkeyLabel"))
        self.gridLayout.addWidget(self.hotkeyLabel, 1, 1, 1, 1)
        self.setHotkeyButton = KPushButton(SettingsWidget)
        self.setHotkeyButton.setObjectName(_fromUtf8("setHotkeyButton"))
        self.gridLayout.addWidget(self.setHotkeyButton, 1, 3, 1, 1)
        self.clearHotkeyButton = KPushButton(SettingsWidget)
        self.clearHotkeyButton.setObjectName(_fromUtf8("clearHotkeyButton"))
        self.gridLayout.addWidget(self.clearHotkeyButton, 1, 4, 1, 1)
        self.label_3 = QtGui.QLabel(SettingsWidget)
        self.label_3.setObjectName(_fromUtf8("label_3"))
        self.gridLayout.addWidget(self.label_3, 2, 0, 1, 1)
        self.windowFilterLabel = QtGui.QLabel(SettingsWidget)
        self.windowFilterLabel.setObjectName(_fromUtf8("windowFilterLabel"))
        self.gridLayout.addWidget(self.windowFilterLabel, 2, 1, 1, 1)
        self.setFilterButton = KPushButton(SettingsWidget)
        self.setFilterButton.setObjectName(_fromUtf8("setFilterButton"))
        self.gridLayout.addWidget(self.setFilterButton, 2, 3, 1, 1)
        self.clearFilterButton = KPushButton(SettingsWidget)
        self.clearFilterButton.setObjectName(_fromUtf8("clearFilterButton"))
        self.gridLayout.addWidget(self.clearFilterButton, 2, 4, 1, 1)
        spacerItem = QtGui.QSpacerItem(40, 20, QtGui.QSizePolicy.Expanding, QtGui.QSizePolicy.Minimum)
        self.gridLayout.addItem(spacerItem, 0, 2, 1, 1)

        self.retranslateUi(SettingsWidget)
        QtCore.QMetaObject.connectSlotsByName(SettingsWidget)

    def retranslateUi(self, SettingsWidget):
        SettingsWidget.setWindowTitle(kdecore.i18n(_fromUtf8("Form")))
        self.label.setText(kdecore.i18n(_fromUtf8("Abbreviations:")))
        self.abbrLabel.setText(kdecore.i18n(_fromUtf8("$abbr")))
        self.setAbbrButton.setText(kdecore.i18n(_fromUtf8("Set&")))
        self.clearAbbrButton.setText(kdecore.i18n(_fromUtf8("Clear&")))
        self.label_2.setText(kdecore.i18n(_fromUtf8("Hotkey:")))
        self.hotkeyLabel.setText(kdecore.i18n(_fromUtf8("$hotkey")))
        self.setHotkeyButton.setText(kdecore.i18n(_fromUtf8("Set&")))
        self.clearHotkeyButton.setText(kdecore.i18n(_fromUtf8("Clear&")))
        self.label_3.setText(kdecore.i18n(_fromUtf8("Window Filter:")))
        self.windowFilterLabel.setText(kdecore.i18n(_fromUtf8("$filter")))
        self.setFilterButton.setText(kdecore.i18n(_fromUtf8("Set&")))
        self.clearFilterButton.setText(kdecore.i18n(_fromUtf8("Clear&")))

from PyKDE4.kdeui import KPushButton
