#!/usr/bin/env python
# coding=UTF-8
#
# Generated by pykdeuic4 from abbrsettings.ui on Sun Mar  4 11:39:39 2012
#
# WARNING! All changes to this file will be lost.
from PyKDE4 import kdecore
from PyKDE4 import kdeui
from PyQt4 import QtCore, QtGui

try:
    _fromUtf8 = QtCore.QString.fromUtf8
except AttributeError:
    _fromUtf8 = lambda s: s

class Ui_Form(object):
    def setupUi(self, Form):
        Form.setObjectName(_fromUtf8("Form"))
        Form.resize(571, 350)
        self.verticalLayout_3 = QtGui.QVBoxLayout(Form)
        self.verticalLayout_3.setObjectName(_fromUtf8("verticalLayout_3"))
        self.horizontalLayout_3 = QtGui.QHBoxLayout()
        self.horizontalLayout_3.setObjectName(_fromUtf8("horizontalLayout_3"))
        self.verticalLayout_2 = QtGui.QVBoxLayout()
        self.verticalLayout_2.setObjectName(_fromUtf8("verticalLayout_2"))
        self.abbrListWidget = QtGui.QListWidget(Form)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Ignored, QtGui.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.abbrListWidget.sizePolicy().hasHeightForWidth())
        self.abbrListWidget.setSizePolicy(sizePolicy)
        self.abbrListWidget.setAlternatingRowColors(True)
        self.abbrListWidget.setUniformItemSizes(True)
        self.abbrListWidget.setObjectName(_fromUtf8("abbrListWidget"))
        self.verticalLayout_2.addWidget(self.abbrListWidget)
        self.horizontalLayout_2 = QtGui.QHBoxLayout()
        self.horizontalLayout_2.setObjectName(_fromUtf8("horizontalLayout_2"))
        self.addButton = QtGui.QPushButton(Form)
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.addButton.setIcon(icon)
        self.addButton.setFlat(False)
        self.addButton.setObjectName(_fromUtf8("addButton"))
        self.horizontalLayout_2.addWidget(self.addButton)
        self.removeButton = QtGui.QPushButton(Form)
        self.removeButton.setText(_fromUtf8(""))
        self.removeButton.setIcon(icon)
        self.removeButton.setObjectName(_fromUtf8("removeButton"))
        self.horizontalLayout_2.addWidget(self.removeButton)
        self.verticalLayout_2.addLayout(self.horizontalLayout_2)
        self.horizontalLayout_3.addLayout(self.verticalLayout_2)
        self.verticalLayout = QtGui.QVBoxLayout()
        self.verticalLayout.setObjectName(_fromUtf8("verticalLayout"))
        self.horizontalLayout = QtGui.QHBoxLayout()
        self.horizontalLayout.setObjectName(_fromUtf8("horizontalLayout"))
        self.triggerOnLabel = QtGui.QLabel(Form)
        self.triggerOnLabel.setObjectName(_fromUtf8("triggerOnLabel"))
        self.horizontalLayout.addWidget(self.triggerOnLabel)
        self.wordCharCombo = KComboBox(Form)
        self.wordCharCombo.setEditable(True)
        self.wordCharCombo.setObjectName(_fromUtf8("wordCharCombo"))
        self.horizontalLayout.addWidget(self.wordCharCombo)
        spacerItem = QtGui.QSpacerItem(40, 20, QtGui.QSizePolicy.Expanding, QtGui.QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem)
        self.verticalLayout.addLayout(self.horizontalLayout)
        self.removeTypedCheckbox = QtGui.QCheckBox(Form)
        self.removeTypedCheckbox.setObjectName(_fromUtf8("removeTypedCheckbox"))
        self.verticalLayout.addWidget(self.removeTypedCheckbox)
        self.omitTriggerCheckbox = QtGui.QCheckBox(Form)
        self.omitTriggerCheckbox.setObjectName(_fromUtf8("omitTriggerCheckbox"))
        self.verticalLayout.addWidget(self.omitTriggerCheckbox)
        self.matchCaseCheckbox = QtGui.QCheckBox(Form)
        self.matchCaseCheckbox.setObjectName(_fromUtf8("matchCaseCheckbox"))
        self.verticalLayout.addWidget(self.matchCaseCheckbox)
        self.ignoreCaseCheckbox = QtGui.QCheckBox(Form)
        self.ignoreCaseCheckbox.setObjectName(_fromUtf8("ignoreCaseCheckbox"))
        self.verticalLayout.addWidget(self.ignoreCaseCheckbox)
        self.triggerInsideCheckbox = QtGui.QCheckBox(Form)
        self.triggerInsideCheckbox.setObjectName(_fromUtf8("triggerInsideCheckbox"))
        self.verticalLayout.addWidget(self.triggerInsideCheckbox)
        self.immediateCheckbox = QtGui.QCheckBox(Form)
        self.immediateCheckbox.setObjectName(_fromUtf8("immediateCheckbox"))
        self.verticalLayout.addWidget(self.immediateCheckbox)
        spacerItem1 = QtGui.QSpacerItem(20, 40, QtGui.QSizePolicy.Minimum, QtGui.QSizePolicy.Expanding)
        self.verticalLayout.addItem(spacerItem1)
        self.horizontalLayout_3.addLayout(self.verticalLayout)
        self.verticalLayout_3.addLayout(self.horizontalLayout_3)
        self.kseparator = KSeparator(Form)
        self.kseparator.setObjectName(_fromUtf8("kseparator"))
        self.verticalLayout_3.addWidget(self.kseparator)

        self.retranslateUi(Form)
        QtCore.QMetaObject.connectSlotsByName(Form)

    def retranslateUi(self, Form):
        Form.setWindowTitle(kdecore.i18n(_fromUtf8("Form")))
        self.triggerOnLabel.setText(kdecore.i18n(_fromUtf8("Trigger on:")))
        self.removeTypedCheckbox.setText(kdecore.i18n(_fromUtf8("Remove typed abbreviation")))
        self.omitTriggerCheckbox.setText(kdecore.i18n(_fromUtf8("Omit trigger character")))
        self.matchCaseCheckbox.setText(kdecore.i18n(_fromUtf8("Match phrase case to typed abbreviation")))
        self.ignoreCaseCheckbox.setText(kdecore.i18n(_fromUtf8("Ignore case of typed abbreviation")))
        self.triggerInsideCheckbox.setText(kdecore.i18n(_fromUtf8("Trigger when typed as part of a word")))
        self.immediateCheckbox.setText(kdecore.i18n(_fromUtf8("Trigger immediately (don\'t require a trigger character)")))

from PyKDE4.kdeui import KSeparator, KComboBox
